#include <Wire.h>
#define I2C_DEV_ADDR 0x55

const int pwm[]  = {19, 4};    // PWM pins for motors 1 and 2
const int in1[]  = {2, 16};
const int in2[]  = {15, 17};

void setMotor(int dir, int pwmVal, int pwmPin, int in1Pin, int in2Pin) {
  analogWrite(pwmPin, pwmVal);
  digitalWrite(in1Pin, dir == 1 ? HIGH : LOW);
  digitalWrite(in2Pin, dir == -1 ? HIGH : LOW);
}

void setup() {
  Serial.begin(115200);
  Wire.begin(); // Leader I2C
  for (int k = 0; k < 2; k++) {
    pinMode(pwm[k], OUTPUT);
    pinMode(in1[k], OUTPUT);
    pinMode(in2[k], OUTPUT);
  }
  Serial.println("Open-loop forward/back test starting...");
}

void loop() {
  static unsigned long startTime = 0;
  static bool runOnce = false;
  int pwmVal;

  if (!runOnce) {
    startTime = millis();
    runOnce = true;
  }

  unsigned long elapsed = millis() - startTime;

  if (elapsed < 500) {
    pwmVal = map(elapsed, 0, 500, 0, 255); // ramp up forward
  } else if (elapsed < 1000) {
    pwmVal = map(elapsed, 500, 1000, 255, 0); // ramp down forward
  } else if (elapsed < 1500) {
    pwmVal = -map(elapsed, 1000, 1500, 0, 255); // ramp up backward
  } else if (elapsed < 2000) {
    pwmVal = -map(elapsed, 1500, 2000, 255, 0); // ramp down backward
  } else {
    pwmVal = 0; // stop
  }

  int dir = pwmVal >= 0 ? 1 : -1;
  int pwr = abs(pwmVal);

  // Apply to leader motors
  setMotor(dir, pwr, pwm[0], in1[0], in2[0]);
  setMotor(-dir, pwr, pwm[1], in1[1], in2[1]);

  // Send to follower
  Wire.beginTransmission(I2C_DEV_ADDR);
  Wire.write((int8_t)pwmVal); // motor 3
  Wire.write((int8_t)pwmVal); // motor 4
  uint8_t err = Wire.endTransmission(true);

  Serial.printf("PWM: %d | I2C err: %u\n", pwmVal, err);
  delay(20);
}
